{code.pas

See the bottom of main.pas for the BSD-style license.

}

{$I osops.i}

unit code;

interface

uses classes, qforms, SysUtils, qStdCtrls
;

procedure splashon;
procedure unsplash;
procedure fixfilecaptions;  {should be called by options}
function os_temporary(thisdir:boolean=false):string;
procedure osdirectory;

function usuref(
const TheFormat: string;
const Args:array of const):boolean; overload;

function usuref(const TheFormat: string):boolean; overload;

function usure(s:string):boolean;

function safe_usuref(const TheFormat: string;
const Args:array of const):boolean;

procedure errorf(
const TheFormat: string;
const Args:array of const); overload;

procedure errorf(const TheFormat: string); overload;

procedure ioerrorf(
const TheFormat: string;
const Args:array of const); overload;

procedure ioerrorf(const TheFormat: string); overload;

procedure infof(
const TheFormat: string;
const Args:array of const); overload;

procedure infof(const TheFormat: string); overload;

procedure bugf(
show:boolean;
const TheFormat: string;
const Args:array of const); overload;

procedure bugf(
show:boolean;
const TheFormat: string); overload;


procedure notimpl(s:string = 'feature');

function dummybump(story: string; howmuch : integer = 1):boolean;

function xlatewordstar(var key:WORD; shift:TShiftState):boolean;

function keyname_code(s:string):integer;

function badfilename(name:string; complain:boolean=false):boolean;

procedure make_dummy;

function evarize(s:string; xtrim:boolean = true):string;


procedure evarlist(sl:TstringList; xtrim:boolean = true);

{$ifdef LINUX}
function LXfindOne(const Path: string; Attr: Integer; var F: TSearchRec):
Integer;

function executize(path:string):boolean;

{$endif}

procedure Really_Terminate_Now;

function formguard(f:Tform):boolean;

procedure working_multioff(t:Tlistbox);

procedure mask_fix(var mask:integer; init:boolean = false);

procedure delphi_yield;

procedure purge_the_path;
procedure replace_oldpath;

var

stop_mask_fix:boolean;  //if set, mask_fix'll skip one.

Gnome_quote_evar:integer;   //you have to clear it before calling evarize et
                            //al.

implementation

uses options, splash, main, publics, sbrs, log,
QDialogs,Qcontrols,qt,osinfo
{$ifdef LINUX}
,Libc
,exec
{$endif}
{$ifdef MSWINDOWS}
,windows
{$endif}
,weirdo
,fileutil
;



{****************************************************************************
****************************************************************************}

procedure splashon;
const
{fn : string = 'splash.jpg';  {this is the wrong piccer.}
fn : string = 'splash.png';
dosplash : string = 'do_splash';

begin
  pgmdirize(fn);
  pgmdirize(dosplash);

{  infof('fn: %s; do_splash: %s',[fn,dosplash]); }

  if (not FileExists_unq(dosplash))
  or (not FileExists_unq(fn))
{  or (not Goptions.splash) }
  then
  exit;

  if fsplash = nil then begin
    fsplash := Tfsplash.Create(Application);
    fsplash.WindowState := wsMaximized;

    try
      fsplash.Image1.Picture.Bitmap.loadfromfile(fn);
    except
      on E: exception do
      errorf('Oops seems like we can''t splash "%s" (wrong picture?); error %s',
      [fn,E.message]);
    end;
  end;

  fsplash.bringtofront;

  fsplash.show;

  fsplash.update;

end;

procedure unsplash;
begin
  if fsplash<>nil then begin
    fsplash.close;
    fsplash.hide;
    fsplash.free;
    fsplash := nil;
  end;
end;

{****************************************************************************
****************************************************************************}

procedure fixfilecaptions;
const
fixed : boolean = false;
begin
  if fixed then
  exit;

  namecaption := safe_format('%s (%s,%s)',
  [namecaption,Goptions.sortName.show,Goptions.sortExt.show]);

  extcaption := safe_format('%s (%s,%s)',
  [extcaption,Goptions.sortName.show,Goptions.sortExt.show]);

  sizecaption := safe_format('%s (%s)',
  [sizecaption,Goptions.sortSize.show]);

  timecaption := safe_format('%s (%s)',
  [timecaption,Goptions.sortDate.show]);


end;

{****************************************************************************
****************************************************************************}
const
JGTEMP = 'JGO';
MAXFILE = 8;
MAXEXT = 4;

function ainc(var tname:string):boolean;
var
i:integer;
begin   { this crashes unless tname is correctly formed. }
  i := 7;
  while (i >= 1) do begin
    tname[i] := succ(tname[i]);
    if tname[i] > 'Z' then begin
      tname[i] := 'A';
      dec(i);
    end
    else begin
      result := true;
      exit;
    end;
  end;

  result := false;
end;

{ if no ext, uses the current one. }
function musttemp(ext:string):string;
const                      {   01234567890123 }
tname:string = 'AAAAAAA~.' + JGTEMP;
var
tfile : string;
begin

  if length(ext)>0 then begin
    tfile := extractFname(tname);
    tname := tfile + '.' + ext;
  end;


  while (FileExists_unq(tname)) do begin
    if (not ainc(tname)) then begin
      result := 'toomany#.' + JGTEMP;
      exit;
    end;
  end;

  result := tname;
  Sysutils.DeleteFile(result);    //gets confused by windows unit....
end;


function os_temporary(thisdir:boolean=false):string;
var
olddir:string;
begin

  if thisdir then begin
    result := musttemp(JGTEMP);
    exit;
  end;

  olddir := getcurrentdir;
  chdir(datadir);
  result := datadir+musttemp(JGTEMP);
  chdir(olddir);
end;

procedure osdirectory;
begin
  chdir(datadir);
end;

{****************************************************************************
These should all be in here and call unsplash.
****************************************************************************}
function usure(s:string):boolean;
begin
//  unsplash;
  result :=
{  MessageDlg(s,mtConfirmation,[mbYes,mbNo],0)
  = mrYES; }

  osinfof(s,[0],osinfo_confirm) =
  osinfo_confirm;

end;

procedure errorf(
const TheFormat: string;
const Args:array of const);
begin
  lg('errorf',TheFormat,Args);
  osinfof(TheFormat,args,osinfo_error);
end;

procedure errorf( const TheFormat: string);
begin
  errorf(TheFormat,[0]);
end;

procedure infof(
const TheFormat: string;
const Args:array of const);
begin
  osinfof(TheFormat,args,osinfo_info);
end;

procedure infof(const TheFormat: string);
begin
  infof(TheFormat,[0]);
end;


function usuref(
const TheFormat: string;
const Args:array of const):boolean;
begin
  unsplash;
{  result := usure(safe_format(Theformat,args)); }
  result :=
  osinfof(TheFormat,args,osinfo_confirm)=osinfo_confirm;
end;

function usuref(
const TheFormat: string):boolean;
begin
  result := usuref(TheFormat,[0]);
end;

function safe_usuref(const TheFormat: string;
const Args:array of const):boolean;
begin
{$ifdef NOPE}
  val := //MessageDlg(safe_format(Theformat,Args),mtConfirmation,[mbYES,mbNO],0);

  MessageDlg(Theformat,mtConfirmation,[mbYES,mbNO],0);
  result := val = mrYES;
{$endif}
//just right. DOESN'T crash and claim drive A: needs a disk at odd moments.
  result :=
  Application.MessageBox(safe_format(TheFormat,args),'Confirmation',
  [smbYES,smbNO],smsWARNING,smbNO)
  = smbYES;
end;

procedure bugf(
show:boolean;
const TheFormat: string;
const Args:array of const);

begin
  if show then
  osinfof(Theformat,args,osinfo_info);
end;

procedure bugf( show:boolean; const TheFormat: string);
begin
  bugf(show,TheFormat,[0]);
end;

procedure notimpl(s:string = 'feature');
begin
  errorf('Sorry; %s not implemented',[s]);
end;

procedure ioerrorf(
const TheFormat: string;
const Args:array of const);
begin
  errorf('%s (%s)',
  [safe_format(TheFormat,Args),sysErrorMessage(getlasterror)]);
end;

procedure ioerrorf( const TheFormat: string);
begin
  ioerrorf(TheFormat,[0]);
end;


function dummybump(story:string; howmuch : integer = 1):boolean;
begin
  result := false;  {don't give up.}
end;

function xlatewordstar(var key:WORD; shift:TShiftState):boolean;
begin
  result := false;

  if (not (ssctrl in shift)) or (key=key_Control) then
  exit;

  result := true;

  case key of
    key_E: key := key_Up;
    key_X: key := key_Down;

    key_R: key := key_Pageup;
    key_C: key := key_Pagedown;
  else
    result := false;
  end;


end;


{****************************************************************************
note that the alt shift nonsense will be encoded somehow separately.
****************************************************************************}
type

kn = record
  s:string;
  k:integer;
end;

const

keys : array[1..16] of kn = (
  ( s: 'F1' ; k: Key_F1     )   {don't let him assign f1. // what r u, some
                                kind of nazi? Tue 8/06/02 4:39 pm}
 ,( s: 'F2' ; k: Key_F2     )
 ,( s: 'F3' ; k: Key_F3     )
 ,( s: 'F4' ; k: Key_F4     )
 ,( s: 'F5' ; k: Key_F5     )
 ,( s: 'F6' ; k: Key_F6     )
 ,( s: 'F7' ; k: Key_F7     )
 ,( s: 'F8' ; k: Key_F8     )
 ,( s: 'F9' ; k: Key_F9     )
 ,( s: 'F10'; k: Key_F10    )
 ,( s: 'F11'; k: Key_F11    )
 ,( s: 'F12'; k: Key_F12    )

 ,( s: {'+'} eq_pl_string; k: jgok_eq_pl {Key_equal}  ) {it's a little
 complicated.... and just got more Tue 8/12/03 10:43 am.}
 ,( s: 'ESC'; k: key_escape )
 ,( s: 'FileEnter'; k: Key_Return )
 ,( s: 'Insert';    k: Key_Insert )


);


function keyname_code(s:string):integer;
var
i:integer;
begin
  for i:=low(keys) to high(keys) do begin
    if CompareText(s,keys[i].s)=0 then begin
      result := keys[i].k;
      exit;
    end;
  end;
  result := -1;
end;


function badfilename(name:string; complain:boolean=false):boolean;
begin
  result :=
  (pos(slashchar,name)>0)
{$ifdef MSWINDOWS}
  or (pos(':',name)>0)
  or (pos('/',name)>0)
{$endif}
  ;
  if complain and result then
  errorf('"%s" not allowed',[name]);
end;

procedure make_dummy;
begin
  Gcheck := dummybump;
end;

{exchange environment variables enclosed within emarks for the thing. two
emarks == one emark in result. if xtrim, trims leading trailing spaces. Honors
psuedo environment variables from Owenshow like _$_osdir.}
{$define PGM_DIR}   //depends on quote_dir.
{$define QUOTE_DIR}
function evarize(s:string; xtrim:boolean = true):string;
const
emark = '%';

GNOME_STRING = '_$_gnome_squote';

{$ifdef QUOTE_DIR}
DIR_STRING = '_$_osdir';
{$endif}
{$ifdef PGM_DIR}
PGM_STRING = '_$_pgmdir';
DAT_STRING = '_$_datdir';

type
Tos_string = ( nada, currentdir, programdir, data_dir );
{$endif}

var
{$ifdef PGM_DIR}
dirkind : Tos_string;
{$endif}
i:integer;
inEmark : boolean;
evar,
estring, d:string;
{$ifdef QUOTE_DIR}
dirstring:string;
x:integer;
{$endif}


  function processemark:boolean;
  begin
    result := true;
    if inEmark then begin
      if s[i]=emark then begin
        if (length(estring)>0)
        and (es(s,i+1)=emark)
        then begin
{he wants an emark *within* an emark expression.... Fat lot of good it'll do
him 'though but whatever...}
          estring :=
          estring + emark;
          inc(i);           //not kosher.
          exit;
        end;
        inemark := false;
        if length(estring)<1 then begin
          d := d + emark       //he wanted an emark, foolish fellow.
        end else begin
          evar := getEnvironmentvariable(estring);
          if length(evar)>0 then
          d := d + evar
          else begin
          //wasn't an e variable, let's see if it's one of ours.
//                                123456
            dirkind := nada;  //should've been here always Wed 10/10/2007
                              //12:37 pm.

            if copy(estring,1,6)='_$_dt:' then begin
              d := d + FormatDateTime(copy(estring,7,5000),now);
            end else

            if estring=GNOME_STRING then begin
              d := d + '''';
              inc(Gnome_quote_evar);
            end else

{$ifdef QUOTE_DIR}
{$ifdef PGM_DIR}
            if      copy(estring,1,length(DIR_STRING))=DIR_STRING then
            dirkind := currentdir
            else if copy(estring,1,length(PGM_STRING))=PGM_STRING then
            dirkind := programdir
            else if copy(estring,1,length(DAT_STRING))=DAT_STRING then
            dirkind := data_dir;

            if dirkind<>nada

{$else}
            if copy(estring,1,length(DIR_STRING))=DIR_STRING
{$endif}
            then begin
//now we'll let him just put in some drivel at the end pretty-much randomly.
//"s, /. $.

{$ifdef PGM_DIR}
              if dirkind=programdir then
              dirstring := pgmdir(false)
              else
              if dirkind=data_dir then
              dirstring := fdeslash(datadir)
              else
{$endif}
              dirstring := thisdir_string(false);

              if pos('/',estring)>0 then    //wants slashed directory.
              dirstring :=
{$ifdef PGM_DIR}
              fslash(dirstring)
{$else}
              thisdir_string
{$endif}
              ;

{$ifdef MSWINDOWS}
              if pos('!',estring)>0 then
              dirstring := shortify(dirstring);
{$endif}

              x := pos('"',estring);
              if (x>0) then begin     //wants quoting.
                if (position(x+1,'"',estring)>0) then
                //two of them; always quote.
                dirstring := quotate(dirstring)
                else
                //one of them; maybe quote.
                dirstring := quotate(dirstring,true);
              end;

              d := d + dirstring; //stuff the resulting directory into
                                  //destination.

            end;
            //else we add nothing to d, and the %blah blah% disappears.
{$else}
            if estring='_$_osdir' then
            d := d + thisdir_string(false)
            //see note below about thisdirectory...
            else
            if (estring='_$_osdir/')
{            or (estring='_$_oddir\')  }
{well I guess the linuxoids win this one; the backslash version doesn't work!}
            then
            d := d +
//            thisdirectory //side-effect of changing to thisdir!
            thisdir_string
            ;
            { else it just disappears }
{$endif}
          end;
        end;
      end else begin
        estring := estring + s[i];
      end
    end else begin
      if s[i]=emark then begin
        estring := '';
        inemark := true;
      end else begin
        //not emark, and not inemark.
        result := false;
      end;
    end;

  end;

begin
  d := '';

  inEmark := false;

  for i:=1 to length(s) do begin
    if not processEmark then
    d := d + s[i];
  end;

  if xtrim then
  result := trim(d)
  else
  result := d;

end;

procedure evarlist(sl:TstringList; xtrim:boolean = true);
var
i : integer;
begin
  for i:=0 to sl.count-1 do
  sl[i] := evarize(sl[i],xtrim);
end;

{$ifdef LINUX}
{****************************************************************************
The idea is in Linux *? are legal file characters, so this'll find a single
file assuming *? are real characters. Note that I won't honor any of this
fclose stuff or whatever.
****************************************************************************}

function LXfindOne(const Path: string; Attr: Integer; var F: TSearchRec):
Integer;
var
name : string;
begin
  result := findfirst(path,attr,F);
  if result<>0 then
  exit;
//not found anyway.

  name := ExtractFileName(path);

  if notwild(name) then
  exit; //found, not wild.

//found, but it's got wild cards, so we'll just go forever
//looking some more. result == 0.

  while findnext(F)=0 do begin
    if name=f.name then
    exit;   //with the 0 result.
  end;

//no esta aqui. Caller must close -- just so it works like findfirst.
  result := -1;

end;

{$endif}

procedure Really_Terminate_Now;
begin
  Application.free;       //these are amazing things.
  Application.Terminate;  //THAT does the job!
{****************************************************************************
Wed 8/18/2004 6:08 pm. I mean I doubt it's a great idea, but it stops the
thing *NOW* which is presumably what one had in mind.... You can see in the
assembler code that Terminate checks if application==0, and does something for
that -- calls Application.quit as it happens, which I would've done if I
could've....
****************************************************************************}
end;

{$ifdef LINUX}
{if path doesn't exist, return false.
 if path exists, and isn't an executable, try to make it an executable. If
 that fails, return false. Otherwise, return true.}
function executize(path:string):boolean;
var
mode : __mode_t;
begin
  result := false;

  if not FileExists_unq(path) then
  exit;

  mode := OS_getmode(path);

  if ((mode and (S_IXUSR or S_IXGRP or S_IXOTH))=0) and
  (chmod(pchar(path),mode or (S_IXUSR or S_IXGRP or S_IXOTH))<>0)
  then
  exit;

  result := true;

end;
{$endif}

function formguard(f:Tform):boolean;
const
TOOCLOSE = 40;
FIX = 80;
begin
  result := false;
  if (notinited>0)
  or (not Goptions.formguard) then
  exit;

  with f do begin
    if left > (screen.width-TOOCLOSE) then begin
      left := screen.width - FIX;
      result := true;
    end else if (left+width) < TOOCLOSE then begin
      left := -width + FIX;
      result := true;
    end;

    if top > (screen.height-TOOCLOSE) then begin
      top := screen.height - FIX;
      result := true;
    end else if top < 0 then begin
      top := 0;     //can only move top.
      result := true;
    end;
  end;
end;

procedure working_multioff(t:Tlistbox);
var
i:integer;
begin

  with t do
  if multiselect and (selcount>1) then begin
    for i := 0 to items.count-1 do begin
      if selected[i]
      and (i<>itemindex) then
      selected[i] := false;
    end;
  end;
  t.multiselect := false;

end;

{****************************************************************************
Tue 1/04/2005 7:35 pm. stuff that uses a mask and count to check with the user
might call this thing whenever it thinks it's time to check the user, and
it'll adjust the mask if it's too long. You should init this thing at the
beginning of your scan or whatever, or it'll definitely be wrong. So I'm only
doing this with describe, because it bit me. Others as needed. // Wed
10/12/2005 10:47 am and so many have joined! Typically, forgetting the init
stage.
****************************************************************************}

procedure mask_fix(var mask:integer; init:boolean = false);
const
lasttime : int64 = 0;
//TOOLONG = 3000; //milliseconds.
//TOOLONG = 1000;
MINMASK = $3;

var
interval, thistime : int64;
begin
  thistime := Get_ms;

{.$define DBGMASK}
  interval := thistime-lasttime;

  if ((interval) > Goptions.fixtoolong)
  and (lasttime<>0)
  and (not Goptions.dontfixcheck)
  and (not init)
  and (not stop_mask_fix)
  then while
  (interval>Goptions.fixtoolong)
  and (mask>MINMASK)
  do
  begin
    mask := mask shr 1;
    interval := interval div 2;
{$ifdef DBGMASK}
    dbg_nolog('mask (%X) SHR',[mask]);
{$endif}
  end;

  lasttime := thistime;
  stop_mask_fix := false;

end;

procedure delphi_yield;
const
here : boolean = false;
var
wason : boolean;
begin

  if here then
  exit;

  here := true;

  wason := Gidle_off;

  Application.HandleMessage;
  Application.ProcessMessages;

  Gidle_on(wason);

  here := false;
end;

{****************************************************************************
****************************************************************************}


var
oldpath:string;

procedure replace_oldpath;
begin
  if oldpath='' then
  exit;

{$ifdef MSWINDOWS}
  SetEnvironmentVariable('PATH',pchar(oldpath));
{$endif}

{$ifdef LINUX}
  setenv('PATH',pchar(oldpath),1);
{$endif}

end;


{****************************************************************************
****************************************************************************}

procedure purge_the_path;
var
p : TstringList;
i,j  : integer;
kill : boolean;
separator,
newpath : string;
begin

  if Goptions.pathpurge.count=0 then
  exit;

  oldpath := getenvironmentvariable('PATH');
  //save it.

  p := pathlist(true);

  i := 0;
  while i < p.count do begin

    kill := false;

    for j := 0 to Goptions.pathpurge.count-1 do begin
      if
{$ifdef MSWINDOWS}
      pos_nocase
{$endif}
{$ifdef LINUX}
      pos
{$endif}
      (Goptions.pathpurge[j],p[i])>0 then begin
        kill := true;
        break;  // for j.
      end;
    end;

    if kill then
    p.delete(i)
    else
    inc(i);

  end;

  newpath := '';
  separator := '';
  for i := 0 to p.count-1 do begin
    newpath := newpath + separator + p[i];
    separator := sepchar;
  end;

  freeandnil(p);

{$ifdef MSWINDOWS}
  SetEnvironmentVariable('PATH',pchar(newpath));
{$endif}

{$ifdef LINUX}
  setenv('PATH',pchar(newpath),1);
{$endif}

end;

{****************************************************************************
****************************************************************************}

initialization

make_dummy;

end.


