{fileutil.pas

See the bottom of main.pas for the BSD-style license.

}

{$I osops.i}

unit fileutil;

interface

uses sysutils, classes, publics
{$ifdef LINUX}
,Libc
{$endif}
;

type
  CantMove_Exception = class(EStreamError);
  LX_cantgetmodException = class(EStreamError);
  NoRoom_Exception   = class(EStreamError);
  NoDir_Exception    = class(EStreamError);
  move_delete_Exception = class(EStreamError);

{****************************************************************************
So the EFOpenError etc. exceptions are defined somewhere like
Source\Rtl\Common\Classes.pas
****************************************************************************}
{$ifdef LINUX}
To_statresult = record
  buf:TStatBuf;
  errno:integer;
end;

{$endif}


function os_fDateTime(const sSrc: string): TDateTime;
function os_fSize(const sSrc: string): int64;
function os_fHasAttr(const sSrc: string; Attr: Word): Boolean;
function OS_readonly(s:string):boolean;

function os_fCopy(const sSrc, sDst: string; SourceSize:int64 = -1):boolean;
function os_fMove(const sSrc, sDst: string; SourceSize:int64 = -1
{$ifdef LINUX}
; os_deleting : boolean = false
{$endif}
):boolean;

{$ifdef LINUX}
function OS_getmode(sSrc:string):mode_t;
function OS_getstat(sSrc:string):To_statresult;
function OS_readlink(spath:string):string;
{$endif}

function os_hCopyBlock(Source,Dest,BytesToCopy:integer;
all:boolean=false):boolean;

{$ifdef MSWINDOWS}
procedure OS_SetArc(s:string);
{$endif}

var

Glast_copy_error : string;
{****************************************************************************
like the miserable DOS global error; caller has to clear it or error'll
linger-on.
****************************************************************************}

implementation

uses QConsts, sbrs, code, {options,} files
{$ifdef MSWINDOWS}
,RTLconsts
,weirdo
{$endif}
,options
;

{$ifdef LINUX}
{****************************************************************************
Mon 9/12/2005 1:32 pm. getstat was failing in strange ways for nfs-mounted
files. So I did the trick with initializing ctime and checking it, on the
theory that maybe the error return wasn't telling the truth -- and that seemed
to fix it. So then I did the To_statresult structure, so I could recover the
error number and see what is was thinking about. However it was always zero
(no error). My guess? ... Who knows? Maybe it didn't like

  if lstat(Pchar(sSrc),result)<>0

(i.e. when result was just TStatBuf) for some mysterious reason; maybe
embedding the TStatBuf into another structure somehow made it happier.
Whatever; I'm leaving the structure just in case I want to do something like
show symbolic link targets or something.

(Changing "stat()" to "lstat()" stopped other miscellaneous failures on
nfs-mounted files.)

TSTATBUF DEFINTION

type
  _stat = record
    st_dev: __dev_t;                     Device.
    __pad1: Word;
    st_ino: __ino_t;                     File serial number.
    st_mode: __mode_t;                   File mode.
    st_nlink: __nlink_t;                 Link count.
    st_uid: __uid_t;                     User ID of the file's owner.
    st_gid: __gid_t;                     Group ID of the file's group.
    st_rdev: __dev_t;                    Device number, if device.
    __pad2: Word;
    st_size: __off_t;                    Size of file, in bytes.
    st_blksize: __blksize_t;             Optimal block size for I/O.
    st_blocks: __blkcnt_t;               Number 512-byte blocks allocated.
    st_atime: __time_t;                  Time of last access.
    __unused1: LongWord;
    st_mtime: __time_t;                  Time of last modification.
    __unused2: LongWord;
    st_ctime: __time_t;                  Time of last status change.
    __unused3: LongWord;
    __unused4: LongWord;
    __unused5: LongWord;
  end;
  |.$EXTERNALSYM stat| // Renamed due to conflict with stat function
  TStatBuf = _stat;
  PStatBuf = ^TStatBuf;


****************************************************************************}

function OS_getstat(sSrc:string):To_statresult;
begin
  //  result.buf.st_ctime := -1;

  result.errno := lstat(Pchar(sSrc),result.buf);

  if (result.errno<>0) // and (result.buf.st_ctime=-1)
  then
  raise LX_cantgetmodException.CreateFmt('Couldn''t stat(%s)',[sSrc]);
end;

function OS_getmode(sSrc:string):mode_t;
begin
  result := OS_getstat(sSrc).buf.st_mode;
end;

{****************************************************************************
< Read the contents of the symbolic link PATH into no more than
   LEN bytes of BUF.  The contents are not null-terminated.
   Returns the number of characters read, or -1 for errors.  >
function readlink(PathName: PChar; Buf: PChar; Len: size_t): Integer; cdecl;
<$EXTERNALSYM readlink>
****************************************************************************}

function OS_readlink(spath:string):string;
const
BSIZE = _POSIX_PATH_MAX+3;
var
buffer : array[0 .. BSIZE] of char;
readlink_result : integer;
begin
  try
    readlink_result := readlink(Pchar(spath),@buffer,BSIZE);
  except
    on exception do begin
      readlink_result := -1;
    end;
  end;

  if readlink_result>=0 then begin
    result := buffer;
    setlength(result,readlink_result);
  end else
  result := '';

end;

{****************************************************************************
****************************************************************************}
{$endif}

const
{$ifdef LINUX}
  SFCreateError = 'Probably cannot create file %s'; {it's in the WIN QConsts}
  SFOpenError = 'Probably cannot open file %s';     {ditto}
{$endif}
  SFMinusOneBytes = '-1 bytes copied to %s; source permissions?';

{****************************************************************************
****************************************************************************}

function os_fCopy(const sSrc, sDst: string; SourceSize:int64 = -1):boolean;
var
  CopyBuffer: Pointer; { buffer for copying }
  byteswrote,
  totalcopied,
  BytesCopied: int64;
  Source, Dest: Integer; { handles }
  Len: Integer;
  Destination: TFileName; { holder for expanded destination name }

const
{ ChunkSize: Longint = 8192; { copy in 8K chunks }
  ChunkSize: Longint = $80000; {let's speeed it up a bit}

  function xGcheck:boolean;
  begin

    if BytesCopied=0 then
    result := Gcheck(sDst,0)
    else
    result := Gcheck(safe_format('"%s" + %s => %s',
    [sDst,
    numbercommas(BytesCopied),numbercommas(TotalCopied)]),
    BytesCopied);

    if result then
    raise EUserAnnoyed.createFmt
    ('user cancelled "%s" copy to "%s"',[sSrc,sDst]);

  end;

  procedure closeSrcDst;
  begin
    if source>=0 then
    FileClose(source);
    source := -1;

    if dest>=0 then
    FileClose(dest);
    dest := -1;
  end;

  procedure offdestination;
  begin
    closeSrcDst;
    DeleteFile(Destination);
  end;

begin
  result := false;
  totalcopied := 0;
  bytescopied := 0; {for Gcheck}
  source := -1;
  dest := -1;

  Destination := ExpandFileName(sDst); { expand the destination path }

  if os_fHasAttr(Destination, faDirectory) then { if destination is a directory... }
  begin
    Len :=  Length(Destination);
    { ...clone file name }
    if Destination[Len] = slashchar then
    Destination := Destination + ExtractFileName(sSrc)
    else
    Destination := Destination + slashchar + ExtractFileName(sSrc);
  end;

  GetMem(CopyBuffer, ChunkSize); { allocate the buffer }

  try

    if not DirectoryExists(ExtractFilePath(Destination)) then
    raise NoDir_Exception.CreateFmt(
    'Destination directory "%s" doesn''t exist',
    [ExtractFilePath(Destination)]);

    if (SourceSize>0) and
{$ifdef LINUX}
{linux is too stupid to realize we meant a possibly-nonexistent file path. Of
course I suppose it could be ambiguous in Linux Land, but not in winders....}
    (DiskBytes(ExtractFilePath(Destination))<SourceSize)
{$endif}
{$ifdef MSWINDOWS}
    (DiskBytes(Destination)<SourceSize)
{$endif}
    then
    raise NoRoom_Exception.CreateFmt('No room for "%s"',[Destination]);

    Source := FileOpen(sSrc, fmShareDenyWrite); { open source file }
    if Source < 0 then
    raise EFOpenError.CreateFmt(SFOpenError, [sSrc]);

{$ifdef LINUX}
    Dest := open(Pchar(Destination),O_CREAT or O_WRONLY or O_TRUNC,
    os_getmode(sSrc) or S_IWUSR);
{$endif}

{$ifdef MSWINDOWS}
    if Goptions.delete_destination then
    DeleteFile(Destination);
    Dest := FileCreate(Destination); { create output file; overwrite existing }
{$endif}
    if Dest < 0 then raise EFCreateError.CreateFmt(SFCreateError, [Destination]);
    try
      repeat

{ I moved it here Thu 7/29/2004 2:10 pm so if copies to/from a diskette, say,
you don't have to wait 2 minutes before the copy form shows some data. // but
then Tuesday, September 7, 2004 4:29 pm I realized I have to do it once at
the end so it doesn't show an incomplete copy during multiple copies under
some conditions..}
        xGcheck;
        BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize); { read chunk }
        if BytesCopied<0 then begin
        {the error isn't docced, but I got it trying to copy an XP file
        with wrong cacls to Linux. Thursday, March 11, 2004 6:34 pm.}
          offdestination;
          raise EFCreateError.CreateFmt(SFMinusOneBytes,[Destination]);
        end;
        if BytesCopied > 0 then begin     {if we read anything... }
          byteswrote :=
          FileWrite(Dest, CopyBuffer^, BytesCopied); { ...write chunk }
          inc(TotalCopied,BytesCopied);

{missing until Friday, July 30, 2004 5:42 pm!}
          if byteswrote<>BytesCopied then
          raise EWriteERror.createFmt
          ('copy of %d bytes from "%s" to "%s" wrote %d bytes',
          [BytesCopied,sSrc,sDst,byteswrote]);

        end;
      until BytesCopied < ChunkSize; { until we run out of chunks }

      xGcheck;  //one at the end

      result := true; {the good end.}
    finally   //inside try.
      closeSrcDst;
      {missing until Thursday, March 11, 2004 6:44 pm!}
    end;
  except  //outside try.
    on e: exception do begin
      offdestination;
      Glast_copy_error := e.message;
    end;
  end;

  FreeMem(CopyBuffer, ChunkSize); { free the buffer }

  closeSrcDst; { close files. I've been sloppy so just do it.}

  if result then {missing until Thursday, March 11, 2004 6:46 pm!}
  FileSetDate(destination,FileAge(sSrc));


end;

{****************************************************************************
****************************************************************************}


{ os_fMove procedure }
{
  Moves the file passed in sSrc to the directory specified in DestDir.
  Tries to just rename the file.  If that fails, try to copy the file and
  delete the original.

  Raises an exception if the source file is read-only, and therefore cannot
  be deleted/moved.
}

{$ifdef MSWINDOWS}
{XP apparently will execute the file-copy-instead-of-move operation all
on its own! But I don't *want that*!}
function samedrive(const sSrc, sDst: string):boolean;
begin
  result :=
  lowercase(es(sSrc,1))=
  lowercase(es(sDst,1));
end;
{$endif}

{.$define MOVE_RO}  //just a test to make sure it'll still fail if for
  //unknown reasons copy works but source delete fails in move/copy.

function os_fMove(const sSrc, sDst: string; SourceSize: int64  = -1
{$ifdef LINUX}
; os_deleting : boolean = false
{$endif}
):boolean;

var
  Destination: string;

begin
  Destination := ExpandFileName(sDst); { expand the destination path }

  try

{$ifndef MOVE_RO}

{$WARNINGS OFF}
    if os_fHasAttr(sSrc, faReadOnly) then  { if it's read-only... }
{$WARNINGS ON}
    raise CantMove_Exception.Create(
    safe_format('%s won''t move r/o "%s"', [OwenShowID,sSrc]));
    { we wouldn't be able to delete it }

{$endif}

    result :=
  {$ifdef MSWINDOWS}
    samedrive(sSrc,Destination) and
  {$endif}
    RenameFile(sSrc,Destination); { try just renaming }

    if result then begin
{$ifdef MSWINDOWS}
      OS_setArc(Destination);
      //make it like the two drive move Thursday, August 26, 2004 4:36 pm.
      //Make that ON W98 -- apparently XP does that already.... Whatever....
{$endif}
      exit; //rename worked.
    end;

    //rename didn't work. Try copy/delete?

    result := os_fCopy(sSrc,Destination,SourceSize);
    { copy it over to destination...}

    if not result then
    exit;   //copy failed; leave os_fCopy Glast_copy error.

    //ok copy worked. try to remove source.

{****************************************************************************
this is bad. Because I want to call this from there. and because it's
unnessary. Oh I see I was hoping to save accidental breakage.
****************************************************************************}

{$ifdef LINUX}
    if (os_deleting) then
    DeleteFile(sSrc)
    else
{$endif}
    result := OS_Delete(sSrc);

    if not result then begin
      raise move_delete_exception.create(
      safe_format('In move, copy to "%s" OK, but couldn''t delete "%s"',
      [Destination,sSrc,Destination]));
    end;

  except
    on e: exception do begin
      result := false;
{$ifdef LINUX}
      if (os_deleting) then
      DeleteFile(Destination)
      else
{$endif}
      os_delete(Destination);
      Glast_copy_error := e.message;
    end;
  end;
end;

{ os_fSize function }
{
  Returns the size of the named file without opening the file.  If the file
  doesn't exist, returns -1.
}

{$ifdef LINUX}
function os_fSize(const sSrc: string): int64;
var
  r: To_statresult;
begin
  result := -1;

  try
    r := OS_getstat(ExpandFileName(sSrc));
    result := r.buf.st_size;

  except
    ;
  end;

end;
{$endif}

{$ifdef MSWINDOWS}
function os_fSize(const sSrc: string): int64;
var
  f: TSearchRec;
begin
  result := -1;
  try
{$WARNINGS OFF}
    if FindFirst(ExpandFileName(sSrc), faAnyFile, f) = 0 then
    Result := f.finddata.nFileSizeLow
    or (f.finddata.nFileSizeHigh shl 32);
{$WARNINGS ON}
  finally
    {SysUtils.} FindClose(f);
  end;
end;
{$endif}


function os_fDateTime(const sSrc: string): System.TDateTime;
begin
  Result := FileDateToDateTime(FileAge(sSrc));
end;

function OS_GetAttr(s:string):integer;
var
f:TSearchRec;
begin
  if FindFirst(s,0 or faDIRECTORY,f) = 0 then
  result := f.attr
  else
  result := 0;

  findclose(f);
end;

{$ifdef MSWINDOWS}
procedure OS_SetArc(s:string);
begin
{$WARNINGS OFF}
  FileSetAttr(s,OS_GetAttr(s) or faArchive);
{$WARNINGS ON}
end;
{$endif}

function OS_readonly(s:string):boolean;
begin
{$WARNINGS OFF}
  result :=
  (OS_GetAttr(s) and faReadOnly)<>0;
{$WARNINGS ON}
end;

function os_fHasAttr(const sSrc: string; Attr: Word): Boolean;
var
 FileAttr: Integer;
begin
  FileAttr := OS_GetAttr(sSrc) { FileGetAttr(sSrc)};
  if FileAttr = -1 then FileAttr := 0;
  Result := (FileAttr and Attr) = Attr;
end;

{****************************************************************************
****************************************************************************}

function os_hCopyBlock(Source,Dest,BytesToCopy:integer;
all:boolean=false):boolean;
var
  CopyBuffer: Pointer; { buffer for copying }
  BytesCopied: Longint;
  Chunk: Longint;
const
  ChunkSize: Longint = $8000; {let's speeed it up a bit}
begin
  result := false;

  GetMem(CopyBuffer, ChunkSize); { allocate the buffer }
  Chunk := ChunkSize;


  try
    while true do begin

      if (not all) and (BytesToCopy<Chunk) then
      Chunk := BytesToCopy;

{     if Chunk=0 then
      break;   }

      BytesCopied := FileRead(Source, CopyBuffer^, Chunk); { read chunk }

      if all and (BytesCopied=0) then begin
        result := true;
        break;
      end;

      if BytesCopied > 0 then begin     {if we read anything... }
        FileWrite(Dest, CopyBuffer^, BytesCopied); { ...write chunk }
        dec(BytesToCopy,BytesCopied)
      end
      else
      break;            {error exit.}

{      if (BytesToCopy=0) then
      break;   }


{geez this is complicated. Turned-out there was no error exit here. Tch tch.
}

{this is the good exit for !all copy.}
      if BytesToCopy=0 then begin
        result := true;
        break;
      end;

    end;

{    result := true; {the good end.}
  except
    on e: exception do begin
      Errorf('os_hCopyBlock error: %s',[e.message]);
    end;
  end;

  FreeMem(CopyBuffer, ChunkSize); { free the buffer }

end;

{****************************************************************************
****************************************************************************}

end.
