{jdisplay.pas

See the bottom of main.pas for the BSD-style license.

}

{$ifdef MSWINDOWS}
{$define FIX_ORANGE}
{$endif}

unit jdisplay;

interface

uses QExtCtrls, Qgraphics, QStdCtrls;

{$I osops.i}

{$I ospix.i}

{$ifdef ASCIIGraf2}
{$define FAKEG}
{$endif}

type

Tjgodis = class
  private
  im : Timage; {Delphi image on a screen somewhere. assigned by create below,
  called by main xinit.}

  hscroll: TScrollBar;
  cn : Tcanvas;
  Flineheight : integer; {computed from the font by create.}
  background : Tcolor;

  nwidth : integer;

{$ifdef FAKEG}
  gHspace,
  gVmiddle,
  halfgwidth,
  wholegwidth{,
  quartergwidth} : integer;
{$endif}

  ds : boolean;
  unshowncount : integer;

  xstart : integer;

  procedure setbackground(c:Tcolor);
  function getbackground:Tcolor;

  procedure setforeground(c:Tcolor);
  function getforeground:Tcolor;

  procedure TextOut(X, Y: Integer; const Text: WideString);

  public


  function dontshow:boolean;
  {used by dir at least.}

  function show(showit:boolean = true; clear:boolean = true):integer;
  procedure killshowcount;

  constructor Create(image:Timage;h:TScrollBar);
  destructor destroy; override;
  function scrLines:integer;  {return screen height in lines.}
  function scrchars:integer;  {return vague idea of char width}
  procedure wrln(line:integer; s:string);
  procedure iwrln(line:integer; s:string);
{  procedure wrlnf(line:integer; const TheFormat: string;
  const Args:array of const); }
  procedure eln(line:integer);

  property backg: Tcolor read getbackground write setbackground;
  property foreg: Tcolor read getforeground write setforeground;
  property lineheight: integer read Flineheight;

  function yline(ypixel:integer):integer;

  procedure skip(howmuch:integer);

  procedure resethscroll;

  procedure bignote(s:string; selectline:integer; bigger:integer);

  procedure refont;

{  private }

end;

var

JGOdisplay : Tjgodis;

implementation

uses SysUtils, main, publics, types, sbrs, options, fontfun
{$ifdef FIX_ORANGE}
,forms
,windows
{$endif}
;
const

{$ifdef FIX_ORANGE}
DirInvertBack : Tcolor = $8080F0;
//orange doesn't do well in 256-color display.
{$else}
DirInvertBack = $8080F0; {orange? I like this color; it's like OV.}
//moved from publics Wed 5/28/2008 11:30 am.
{$endif}



{$define BRUSH_IS_BACKGROUND}
{****************************************************************************
Thu 1/09/03 5:32 pm. Don't we have fun? Turns out this is necessary for this
code to work in old Delphi non-CLX, where the text kept printing with a white
background. Why did it work in CLX? My guess is the QT toolkit that CLX uses
*actually* paints text transparently, while the windows version uses the
current background aka "brush" color regardless of what the documentation
claims. // I have no idea what he's talking about Monday, June 7, 2004 1:15
pm. // Wed 8/24/2005 4:45 pm. Well that part about the transparent printing
versus background makes sense; how that relates to BRUSH_IS_BACKGROUND I still
don't see. Oh yes; I see "setbackground" below also sets the brush.color  so
we don't have to change it when we're erasing something. The important B_I_B
changed is in "setbackground"; the other conditional change is just a
convenience, since that's that way.

Now if I could just figure-out what "delphi non-CLX" I was thinking-of....
****************************************************************************}

procedure Tjgodis.refont;
var
fixed : boolean;
begin
  Flineheight := cn.font.height;

  nwidth := cn.textWidth('n');

{$ifdef FAKEG}
  fixed := (nwidth=cn.textWidth('m'));

  halfgwidth := Flineheight div 2;
  wholegwidth := halfgwidth * 2;

{  gVmiddle := wholegwidth div 3;  }

  gVmiddle := (wholegwidth * 45 ) div 100;
  {gVmiddle is distance *down* to what we guess is the middle of the
  character, i.e. so the graphic down, right-turn will point at the
  middle of the character.}

  gHspace := (Flineheight * 10) div 100;

  if gHspace=0 then
  gHspace := 1;

  if not fixed then
  inc(gVmiddle);
  {it makes it happy some days...}

  if fLineheight>7 then
  inc(fLineheight,1 {gHspace}   );
  {you have to make it *bigger* so it doesn't write-over the line over it.}


  {it's all guesswork.}

  inc(fLineheight,Gtreelineadjust);
  Gislineadjust := Gtreelineadjust;
{$endif}

end;

constructor Tjgodis.Create(image:Timage;h:TScrollBar);
{$ifdef FIX_ORANGE}
  procedure fixorange;
  var
  bits : DWORD;
  begin
    Bits := GetDeviceCaps(GetDC(Application.Handle), BITSPIXEL);

    if bits<=8 then
    DirInvertBack := clBlack;

  end;
{$endif}

begin
  inherited Create;
  {code goes after inherited create}
  hscroll := h;

  im := image;
  cn := im.canvas;

  backg := unselcolor;
  foreg := clblack;
  refont;

{$ifdef FIX_ORANGE}
  fixorange;
{$endif}

end;

destructor Tjgodis.destroy;
begin
  {code goes before inherited destroy}
  inherited destroy;
end;

function Tjgodis.scrLines:integer;  {return screen height in lines.}
begin
  result := im.height div Flineheight;
end;

function Tjgodis.scrchars:integer;
begin
  result := im.width div nwidth;
end;

{$ifdef BRUSH_IS_BACKGROUND}
procedure Tjgodis.eln(line:integer);
begin
  cn.fillrect(Rect(0,line*Flineheight,im.width,(line+1)*Flineheight));
end;
{$else}
procedure Tjgodis.eln(line:integer);
var
oldcolor : Tcolor;
begin
  oldcolor := cn.Brush.color;
  cn.Brush.color := background;
  cn.fillrect(Rect(0,line*Flineheight,im.width,(line+1)*Flineheight));
  cn.Brush.color := oldcolor;

{$ifdef WHYDOESNTRESIZE}
  form1.debuglabel.caption := safe_format('w: %d; h: %d',
  [im.width, im.height]);
{$endif}
end;
{$endif}

{$ifdef FAKEG}

procedure Tjgodis.wrln(line:integer; s:string);
var
y, xx, x, i:integer;
w:integer;
gend : boolean;

  procedure fixw;
  var
  two : char;
  begin
    two := es(s,i+1);
{    w := ord(s[i])*gmult+ord(two);  }
    w := ord(s[i]) shl gshift + ord(two);
    if two='-' then
    gend := true;

  end;

begin
  if dontshow then
  exit;

  eln(line);
  x := xstart {0};
  y := line*Flineheight;
  gend := false;

  for i:=1 to length(s) do begin

    if (i and 1)=0 then
    continue; {while we're doing the graphix prefix, we skip the odd-
    numbered characters; I don't really understand why.... Oh of course
    because fixw eats them 2 characters at a time, i.e. the "ascii
    graphics" are things like '|-' which fixw detects.}

    if gend then begin
      inc(x,gHspace);
      textout(x,y,copy(s,i,10000));
      exit;
    end;

{otherwise we process "graphics" characters at the beginning of the line, to
get cute directory structure. }

    fixw;

    xx := x;

    case w of
{      wgfntH         : begin
        cn.MoveTo(x,y+gVmiddle);
        inc(x,wholegwidth);
        cn.Lineto(x,y+gVmiddle);
      end;  }
{sadly that graphic didn't acherally exist!}

      wgfntHdown     : begin
        gend := true;
        cn.MoveTo(xx,y+gVmiddle);
        inc(xx,halfgwidth);
        cn.Lineto(xx,y+gVmiddle);
        cn.Lineto(xx,y+Flineheight);
      end;

      wgfntV         : begin
        inc(xx,halfgwidth);
        cn.MoveTo(xx,y);
        cn.LineTo(xx,y+Flineheight);
      end;

      wgfntVright    : begin
        inc(xx,halfgwidth);
        cn.MoveTo(xx,y);              {draw  --}
        cn.LineTo(xx,y+Flineheight);  {draw  --.}
                                      {        |}

        cn.MoveTo(xx,y+gVmiddle);
        xx := x + wholegwidth -1;
        cn.LineTo(xx,y+gVmiddle);
      end;

      wgfntVendright : begin
        inc(xx,halfgwidth);
        cn.MoveTo(xx,y);
        cn.LineTo(xx,y+gVmiddle);
        xx := x + wholegwidth -1;
        cn.LineTo(xx,y+gVmiddle);
      end;

{      wgfntBlank     : begin
      end;  }
    end;
    inc(x,wholegwidth);
  end;
end;

{$else}

procedure Tjgodis.wrln(line:integer; s:string);
begin
  if dontshow then
  exit;

  eln(line);
  textout(xstart,line*Flineheight,s);
end;

{$endif}

procedure Tjgodis.iwrln(line:integer; s:string);
var
oldfore,
oldback:integer;
begin
  if dontshow then
  exit;

  oldfore := foreg;
  oldback := backg;

  foreg := clwhite;
  backg := DirInvertBack {oldfore};
  wrln(line,s);

  foreg := oldfore;
  backg := oldback;

end;

{
procedure Tjgodis.wrlnf(line:integer;
const TheFormat: string; const Args:array of const);
begin
  wrln(line,safe_format(TheFormat,Args));
end;
}

procedure Tjgodis.setbackground(c:Tcolor);
begin
  background := c;
{$ifdef BRUSH_IS_BACKGROUND}
  cn.Brush.color := c;
{$endif}
end;

function Tjgodis.getbackground:Tcolor;
begin
  result := background;
end;

{given the ypixel on the picture, return the line number.}
function Tjgodis.yline(ypixel:integer):integer;
begin
  result :=
  ypixel div Flineheight;
end;

procedure Tjgodis.skip(howmuch:integer);
begin
  xstart := -1 * (howmuch * nwidth);
end;

procedure Tjgodis.setforeground(c:Tcolor);
begin
  cn.font.color := c;
  cn.pen.color := c;
end;

function Tjgodis.getforeground:Tcolor;
begin
  result := cn.font.color;
end;

procedure Tjgodis.resethscroll;
begin
  hscroll.max := im.width div nwidth;
end;

procedure Tjgodis.TextOut(X, Y: Integer; const Text: WideString);
var
rightmost:integer;
begin
  cn.textout(x,y,Text);

  rightmost := cn.TextWidth(text);

  inc(rightmost,x-xstart);

  rightmost := rightmost div nwidth;

  if rightmost>hscroll.max then begin
    hscroll.max := rightmost;
  end;
end;

{.$define PMODE}   //see comments below somewhere.
{.$define BIGCANVAS}

{$ifdef BIGCANVAS}

var
bigimage : Timage;

procedure Tjgodis.bignote(s:string; selectline:integer; bigger:integer);
var
{oldmode : TPenMode; }
xpos,
position : integer;
bigcanvas : Tcanvas;
r : Trect;
const
LITTLEHEIGHT = 35;
BIGHEIGHT = 60;    //was 70. but I made it a capital drive letter.
BIGGERHEIGHT = 120;

begin

  if dontshow then
  exit;

  if (bigimage = nil) then begin
    bigimage := Timage.create(form1);

    bigimage.transparent := true;

    bigimage.left := form1.dirimage.left;
    bigimage.top := form1.dirimage.top;

    bigimage.bringtofront;

    bigimage.picture.bitmap := Tbitmap.create;

    with bigimage.picture.bitmap.canvas do begin
      font.assign(cn.font);
      font.pitch := fpVariable;
      font.name :=
  {$ifdef MSWINDOWS}
      'Arial'
  {$endif}
  {$ifdef LINUX}
      'Helvetica'
  {$endif}
      ;

      brush.color := clWhite;

    end;
  end;

  bigcanvas := bigimage.picture.bitmap.canvas;

//convert it to a position.
  selectline := selectline * flineheight;

  if length(s)>2 then begin
    bigcanvas.font.height := LITTLEHEIGHT;
    bigcanvas.font.color := clTeal;
  end else begin
    if bigger then begin
      bigcanvas.font.height := BIGGERHEIGHT;
      bigcanvas.font.color := $7F; //dark red.
    end else begin
      bigcanvas.font.height := BIGHEIGHT;
      bigcanvas.font.color := clTeal;
    end;
  end;

  if selectline > (bigcanvas.font.height+flineheight) then
  position := 0
  else
  position := bigcanvas.font.height + flineheight;

  xpos := im.width - bigcanvas.TextWidth(s) - 2;
  if (xpos<0) then
  xpos := 0;

  bigimage.width := im.width;
  bigimage.height := position + bigcanvas.font.height;

  bigcanvas.textout(
  xpos,
  position
  ,s);

//  r := rect(0,0,bigimage.width,bigimage.height);

//  cn.copymode := cmSrcInvert;
//  cn.copyrect(r,bigcanvas,r);

end;


{$else}
procedure Tjgodis.bignote(s:string; selectline:integer; bigger:integer);
var
oldfont : Tfont;
{oldmode : TPenMode; }
xpos,
position : integer;
{$ifdef PMODE}
oldpmode : Tpenmode;
{$endif}
const
LITTLEHEIGHT = 35;
BIGHEIGHT = 60;    //was 70. but I made it a capital drive letter.
BIGGERHEIGHT = 120;

begin

  if dontshow then
  exit;

//convert it to a position.
  selectline := selectline * flineheight;

  oldfont := Tfont.create;
  oldfont.assign(cn.font);
{  oldmode := cn.pen.mode; }

  cn.font.pitch := fpVariable;
  cn.font.name :=
{$ifdef MSWINDOWS}
  'Arial'
{$endif}
{$ifdef LINUX}
  'Helvetica'
{$endif}
  ;

  if bigger=0 then begin
    cn.font.height := BIGHEIGHT;
    cn.font.color := clTeal;
    if length(s)>2 then
    cn.font.height := LITTLEHEIGHT;
  end else if bigger>9 then begin
    cn.font.height := bigger;
    cn.font.color := $7F; //dark red.
  end else begin
    cn.font.height := BIGGERHEIGHT;
    cn.font.color := $7F; //dark red.
  end;

  if selectline > (cn.font.height+flineheight) then
  position := 0
  else
  position := cn.font.height + flineheight;

  ;


//  if position+cn.font.height < im.height then
{don't print it if no room} {oh why not...}

{  cn.pen.mode := pmWhite; }
{seems to have no effect. There's a "fontpen" which is probably it, but
indecipherable. // you have to retrieve it from somewhere; that is,
font.fontpen is some kind of handle or something you then  use to retrieve
a pen object from somewhere. Google is silent, almost (!) //see next comment
circa PMODE}
//  cn.pen.mode :=
//  pmMergePenNot;


  xpos := im.width - cn.TextWidth(s) - 2;
  if (xpos<0) then
  xpos := 0;

{$ifdef PMODE}
  oldpmode := cn.pen.mode;
{****************************************************************************
If I do this without saving and restoring cn.pen.mode, then the lines don't
draw! But presumably this is the wrong thingey; the QT font has FontPen: QPenH
handle property:

Use FontPen to return the instance pointer to the QPen object used by the font
object to render the text on a canvas.

//Ok I just checked it out, and the QT pen DOES NOT have XOR style thingeys
like brushes or something. //And apparnetly qbrush doesn't got those neither.
Just a simple innocent thingey knowing nothing of xors. But then again,
presumably pmXOR had SOME effect, since it erased the lines! Hmmmm.... Well
copymode could copy the thing onto the canvas with various amusing modes, but
it seems like a lot of trouble.... Yes see bigcanvas above; never got anything
to happen....

****************************************************************************}

  cn.pen.mode := pmNOT;
{$endif}

  cn.textout(
  xpos,
  position
  ,s);

{$ifdef PMODE}
  cn.pen.mode := oldpmode;
{$endif}

  cn.font.assign(oldfont);

  oldfont.free;

{  cn.pen.mode := oldmode;  }


end;

{$endif}


{.$define DEBUG_DONTSHOW}

function Tjgodis.dontshow:boolean;
begin
  result := ds;

  if {not} result then begin
  {amazingly it worked ok for a while with the not! werd.}
    inc(unshowncount);
{$ifdef DEBUG_DONTSHOW}
    dbg('usc:%d',[unshowncount]);
{$endif}
  end;
end;

function Tjgodis.show(showit:boolean = true; clear:boolean = true):
integer;
begin
  result := unshowncount;

  if showit then begin
    if clear then
    unshowncount := 0
    else
    inc(unshowncount);  {make sure system knows need redisplay?}
    ds := false;
  end else begin
    ds := true;
  end;

end;

procedure Tjgodis.killshowcount;
begin
  unshowncount := 0;
end;


initialization

finalization

{$ifdef BIGCANVAS}
//FreeAndNil(bigimage);
//no the image'll be freed by form1?
{$endif}

FreeAndNil(jgodisplay);

//if jgodisplay<>nil  {forgot until Fri 11/29/02 2:26 pm}
//then begin
//  jgodisplay.free; {forgot until Thursday, November 21, 2002 11:51 am}
//  jgodisplay := nil;  {forgot until Wednesday, December 17, 2003 8:48 am}
//end;




end.
