unit layout;

{
See the bottom of main.pas for the BSD-style license.
}

{$I osops.i}

{$define STUPID_KYLIX defined(LINUX)}
//q.v. in a comment somewhere.

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QButtons, QExtCtrls;

type
  Tflayout = class(TForm)
    CancelButton: TBitBtn;
    OKButton: TBitBtn;
    BitBtn_Help: TBitBtn;
    ListBox_Layout: TListBox;
    Edit: TEdit;
    Label_new: TLabel;
    Timer: TTimer;
    RadioButton_Recall: TRadioButton;
    RadioButton_Save: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimerTimer(Sender: TObject);
    procedure ListBox_LayoutDblClick(Sender: TObject);
    procedure BitBtn_HelpClick(Sender: TObject);
    procedure ListBox_LayoutKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BitBtn_SaveRecallClick(Sender: TObject);
    procedure RadioButton_RecallClick(Sender: TObject);
  private
    { Private declarations }
    recall_main_now,
    save_it:boolean;
    who : string;
    procedure fix_list;
    procedure fiddle_saveit;
  public
    { Public declarations }
  end;

var
  flayout: Tflayout;

type

s_r_type = (s_r_save, s_r_recall, s_r_nada);


function save_recall(recall_main:boolean; var s:string):s_r_type;
function save_restore_filename(rut:string):string;
//these encodes like "main" to "layout_main.ini"

implementation

{$R *.xfm}

uses options, sbrs, code, qt, fontfun,
weirdo
, main;

const
prefix = 'layout_';
save_restore_extension = '.ini';

procedure Tflayout.FormCreate(Sender: TObject);
begin
  autoscroll := true;
end;

function save_recall(recall_main:boolean; var s:string):s_r_type;
begin
  with flayout do begin
    recall_main_now := recall_main;
    save_it := false;         //default to recall.
    flayout.showmodal;
    s := who;
    if (modalresult = mrOK)
{$ifdef STUPID_KYLIX}
    and (who<>'')
{$endif}
     then begin
      if save_it then
      result := s_r_save
      else
      result := s_r_recall;
    end else
    result := s_r_nada;
  end;
end;

procedure Tflayout.fiddle_saveit;
begin
  if save_it then begin
    edit.visible := true;
    label_new.visible := true;
    RadioButton_save.checked := true;
    caption := 'save layout';
  end else begin
    edit.visible := false;
    label_new.visible := false;
    RadioButton_recall.checked := true;
    caption := 'recall layout';
  end;

  edit.text := '';
  who := '';
  Listbox_Layout.setfocus;

end;



procedure Tflayout.FormShow(Sender: TObject);
begin
  fixfonts(flayout);
  formguard(flayout);



  fix_list;


  fiddle_saveit;

  Timer.enabled := recall_main_now;

end;

procedure Tflayout.fix_list;
var
f:TsearchRec;
index, ffresult:integer;


begin
  ffresult := findfirst(
  save_restore_filename('*'),0,f);

  Listbox_Layout.clear;

  while ffresult=0 do begin
    Listbox_Layout.items.add
    (copy(ExtractFName(f.name),length(prefix)+1,5000));
    ffresult := findnext(f);
  end;

  findclose(f);

  Listbox_layout.sorted := true;

  index := Listbox_layout.items.indexof(Goptions.layout_main);
  if index<0 then
  Listbox_layout.items.add(Goptions.layout_main);

  index := Listbox_layout.items.indexof(Goptions.layout_main);
  Listbox_layout.itemindex := index;

end;

{****************************************************************************
****************************************************************************}


procedure Tflayout.CancelButtonClick(Sender: TObject);
begin
  modalresult := mrCancel;
end;

procedure Tflayout.OKButtonClick(Sender: TObject);
begin
  modalresult := mrOK;
end;

function save_restore_filename(rut:string):string;
begin
  result := datadir+prefix+rut+save_restore_extension;
end;


{****************************************************************************
Oops! "`%s" doesn't exist! (Note that the `%s layout doesn't exist until
you've exited the program once, or if you've explicitly saved it)
`a,[who,Goptions.layout_main]

****************************************************************************}


procedure Tflayout.FormClose(Sender: TObject; var Action: TCloseAction);
var
rutname:string;
begin
  if
  (modalresult<>mrOK) then
  exit;

  who := '';

  if edit.text <> '' then
  who := punc_blr(ExtractFName(edit.text))
  else try
    who := Listbox_Layout.items[Listbox_Layout.itemindex];
  except
    on exception do;
  end;

  rutname := who;

  if who<>'' then begin
    who := save_restore_filename(who);

    if (not save_it)
    and (not FileExists_unq(who))
    then begin
{****************************************************************************
Thu 9/21/2006 4:28 pm. Ah modalresult strikes again! See if u do this in
Kylix, it doesn't like it! I thought it was perhaps calling formclose again,
and I tried trapping it so it wouldn't reenter -- but that was stupid (1.)
because it should already not re-entered because the modalresult is no longer
mrOK, and (2.) because I did something else stupid.

So I will test for who<>'' in save_recall up there, which runs this show, and
assign modalresult in formclose no more forever....

But that didn't fix it; Kylix would still hang-up when I tried to recall the
main layout in the normal program, and it *was* that menacing errorf!
Apparently the rule to follow is "don't do ANYTHING in Kylix formclose". ...
Although I suspect it might've been the wicked code in osinfo assigning
something to *its* modalresult, bad bad.... Whatever....

I suspect it's something like, don't do anything in formclose that would call
formclose again; I suspect the Linux CLX calls some routine that then calls
formclose, and *that* routine isn't properly re-entrant or unclean or
something....

So see STUPID_KYLIX conditional def somewhre above.

****************************************************************************}

{$ifndef STUPID_KYLIX}
      modalresult := mrCancel;    //this definitely hangs-up Kylix.
{$endif}


      if not os_off then
      //linux didn't like running this error during startup; and it's really
      //not helpful anyway.
      //wasn't the stupid error anyway! See modalresult nonsense above.

{$ifdef STUPID_KYLIX}

      MessageDlg(safe_format(


'Oops! "%s" doesn''t exist! (Note that the %s layout doesn''t exist until'
+' you''ve exited the program once, or if you''ve explicitly saved it)'
,[who,Goptions.layout_main]

      )


      ,mtError,[mbOK],0);


{$else}

      errorf(                     //this definitely hangs-up Kylix.

'Oops! "%s" doesn''t exist! (Note that the %s layout doesn''t exist until'
+' you''ve exited the program once, or if you''ve explicitly saved it)'
,[who,Goptions.layout_main]

      );

{$endif}

      who := '';

    end;

  end;

end;

procedure Tflayout.TimerTimer(Sender: TObject);
begin
  timer.enabled := false;
  OKButtonClick(nil);
end;

procedure Tflayout.ListBox_LayoutDblClick(Sender: TObject);
begin
  OKButtonClick(nil);
end;

procedure Tflayout.BitBtn_HelpClick(Sender: TObject);
begin
  oshelp('Etc Recall-Save Layout');
end;

procedure Tflayout.ListBox_LayoutKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if shift<>[ssCtrl] then
  exit;

  with ListBox_Layout do
  case key of
    ord('X'): if ItemIndex<(items.count-1) then ItemIndex := ItemIndex +1;
    ord('E'): if ItemIndex>0 then ItemIndex := ItemIndex - 1;
  end;

end;

procedure Tflayout.BitBtn_SaveRecallClick(Sender: TObject);
begin
  save_it := not save_it;
  fiddle_saveit;
end;

procedure Tflayout.RadioButton_RecallClick(Sender: TObject);
const
here : boolean = false;
begin
  if here then
  exit;

  here := true;

  save_it := RadioButton_Save.checked;
  fiddle_saveit;

  here := false;

end;

end.
