{osedir.pas

See the bottom of main.pas for the BSD-style license.

}

{$I osops.i}

{$define FIX_SORT}  //I dunno.

unit osedir;

interface

uses classes, qforms, SysUtils, qStdCtrls, main;

//returns true if it execed.
function edir(thisfv:Tfv):boolean;

implementation

uses options, publics, sbrs, exec, code, files
,Masks
;

{****************************************************************************
          10        20        30        40        50
   1234567890123                 123456789012345678901
 123456789012345678901234567890123456789012345678901234567890
|  1,123,491,968  2-15-02 12:02:20p|badfonts_cpyfiles.psp|
|  9,123,491,968  2-15-02 12:02:20p|badfonts_cpyfiles.psp|
|    12,123,491K  2-15-02 12:02:20p|badfonts_cpyfiles.psp|
|-     1,491,968 12-15-02  2:02:20p|badfonts_cpyfiles.psp|
|          <DIR>  2-15-02  2:02:20p|badfonts_cpyfiles.psp|
|-         <DIR>  2-15-02  2:02:20p|badfonts_cpyfiles.psp|
                 123456789012345678

The "-" denotes a no-longer with us entry.

So the logic is

0. Get all the file names into a list.

1. Read the input directory, output to a temp.

2. If line is proper file record

   1. Check if it's in the directory list; if not, mark it minus.

   2. If it is, update its stats from the file info. Remove it from the list.

3. When finished with that, go through the list emitting the stuff in it, i.e.
at the end of the edir file.

||

****************************************************************************}

const
//POS_MARK1 = 1;
{****************************************************************************
spaces between things and some sizes.
****************************************************************************}
DELETE_SIZE_SPACE = 1;
SIZE_SIZE = 13;
SIZE_TIME_SPACE = 1;
TIME_SIZE = 18;
//TIME_NAME_SPACE = 1;

var
ifile, ofile:integer;

fj:TStringList;
fv:Tfv;

s_sz, m_sz, e_sz:integer;

dir_center:string;

dontseedots : boolean;

{****************************************************************************
****************************************************************************}
procedure xinit;
const
done : boolean = false;
begin
  if done then
  exit;

  done := true;

  s_sz := length(Goptions.edir_tag);
  m_sz := length(Goptions.edir_midtag);
  e_sz := length(Goptions.edir_endtag);

  dir_center := stringOfChar(' ',SIZE_SIZE div 2);

end;


{****************************************************************************
return true if name is not forbidden by options.
****************************************************************************}

{.$define DBG_WILD}

function okname(n:string):boolean;
var
i:integer;

begin
  result := false;

  if

  (pos(Goptions.edir_tag,n)   >0) or
  (pos(Goptions.edir_midtag,n)>0) or
  (pos(Goptions.edir_endtag,n)>0)

  then
  exit;

//do the skipchars.
  for i := 1 to length(Goptions.edir_skip_chars) do begin
    if pos(Goptions.edir_skip_chars[i],n)>0 then
    exit;
  end;

  if (Goptions.edir_skip_list.count>0)
  then begin
    for i := 0 to Goptions.edir_skip_list.count-1 do begin
      if
      MatchesMask(n,Goptions.edir_skip_list[i])
      then
{$ifdef JUST_DO_IT_BELOW_SOMEWHERE}
      begin
        if

        not

        (
        (n[1]='.') and   //it's a leading-dot file.
        dontseedots
        )

        then
        exit;   //with false.
      end;
{$else}
      exit;     //exit for any match.
{$endif}


    end;
  end;

  result := true;

end;


{****************************************************************************
get the files in the current file directory and put them in this list. Keep a
record of their *index* in the object field of the fj[].
****************************************************************************}

procedure getosnames;
var
i:integer;
begin

  with fv do
  for i := 0 to filecount-1 do begin
    if okname( Tos_f(fls.objects[i]) .name
    ) then
    fj.addobject( Tos_f(fls.objects[i]) .name ,TObject(i));
  end;

end;

{****************************************************************************
****************************************************************************}
{$ifndef FIX_SORT}
procedure getosexts;
var
i:integer;

begin
  with fv do
  for i := 0 to filecount-1 do begin
    if okname( Tos_f(fls.objects[i]) .name)
    then
    fj.addobject( safe_format('a%4s',[ Tos_f(fls.objects[i]) .ext ])
    + Tos_f(fls.objects[i]) .name,TObject(i));

{****************************************************************************
see we get the ext+name so it sorts on the ext first, but also the name part.
****************************************************************************}
  end;
end;

{****************************************************************************

****************************************************************************}
procedure exts_names;
var
i:integer;
begin
  with fv do

  for i := 0 to fj.count-1 do
  fj[i] := Tos_f(fls.objects[integer(fj.objects[i])]).name;
//the object apparently remains.

end;
{****************************************************************************
****************************************************************************}
{$endif}    //if FIX_SORT.

function isdir(index:integer):boolean;
begin
  result := Tos_f(fv.fls.objects[index]) .dir;
end;

{****************************************************************************
produce an appropriate edir-style string from the file thingey.
****************************************************************************}
function fv_string(index:integer; plus:boolean = false):string;
var
plusmark,
sz,tm,nm:string;
sizepad, timepad:integer;

begin
  if plus then
  plusmark := '+'
  else
  plusmark := ' ';

  try

    sz := fv.view.items[index].subitems[sizecolumn-1];
    if (sz=slashchar) then
    sz := slashchar +
//    stringOfChar(' ',SIZE_SIZE div 2)   idiot
    dir_center
    ;
    //move it over a little.

    tm := fv.view.items[index].subitems[timecolumn-1];
    nm := Tos_f(fv.fls.objects[index]) .name;

    sizepad := SIZE_SIZE - length(sz);
    if sizepad<0 then
    sizepad := 0;

    timepad := TIME_SIZE - length(tm);
    if timepad<0 then
    timepad := 0;

    result := Goptions.edir_tag+plusmark+spaces(DELETE_SIZE_SPACE)+
    spaces(sizepad) + sz + spaces(SIZE_TIME_SPACE) +
    spaces(timepad) + tm + Goptions.edir_midtag +
    nm
    +
    Goptions.edir_endtag;

  except
    on exception do
    result := '';
  end;

end;


procedure emitfilejunk(s:string; pos_start,pos_end,index:integer);
begin

  hprintf(ofile,
  '%s%s%s'+LNEND,
  [
  copy(s,1,pos_start-1),
  fv_string(index),
  copy(s,pos_end+e_sz,5000)
  ]
  );

end;

{****************************************************************************
1234
  |  987120934

****************************************************************************}

procedure emitdeletedjunk(s:string; pos_start,pos_end:integer);
begin
  if length(s) > (pos_start) then
  s[pos_start+s_sz] := '-';

  hprintf(ofile,'%s'+LNEND,[s]);

end;


{****************************************************************************
1. Read the input directory, output to a temp.

2. If line is proper file record

   1. Check if it's in the directory list; if not, mark it minus.

   2. If it is, update its stats from the file info. Remove it from the list.

   3. But non-dir lines must be passed-through, dolt!

****************************************************************************}

procedure processedir;
var
s,n:string;
index, pos_start, pos_mid, pos_end:integer;
begin
  while (hgets(ifile,s)) do begin
{$ifdef DBG_WILD}

    if pos('345',s)>0 then
    infof('Found 345');
{$endif}
    pos_start := position(1,Goptions.edir_tag,s);
    pos_mid :=   position(pos_start+s_sz,Goptions.edir_midtag,s);
    pos_end :=   position(pos_mid+m_sz,Goptions.edir_endtag,s);
//first, second marker.

    if (pos_start<1)
    or (pos_end<1)
    or (pos_end<1)
    then begin
      hprintf(ofile,'%s'+LNEND,[s]);  //print text comments etc.
      continue;     //the while.
    end;

    n := copy(s,

    pos_mid+m_sz,

    (pos_end) -
    (pos_mid + m_sz)

    );

{$ifdef FIX_SORT}
    if not fj.find(n,index) then
    index := -1;
{$else}
    index := fj.IndexOf(n);
{$endif}

    if
    (index>=0)
    then begin
      emitfilejunk(s,pos_start,pos_end,integer(fj.objects[index]));
      fj.delete(index);   //remove it from the remainders pile.
    end else
    emitdeletedjunk(s,pos_start,pos_end);

  end;

end;

{****************************************************************************
the idea is it should always produce the same order, which is more or less
alphabetical or perhaps extension. But it should put the dirs up front.
****************************************************************************}

{.$define dbg_SEE_SORT}  //debug to output stuff in list.
procedure output_remainders;
var
j, i : integer;
dir : boolean;
begin
//do directories first.
  for j := 1 to 2 do
  for i := 0 to fj.count-1 do begin
    dir := isdir(integer(fj.objects[i]));

    if
    ((j=1) and (dir))
    or
    ((j=2) and (not dir))
    then begin
{$ifdef dbg_SEE_SORT}
      hprintf(ofile,'%s'+LNEND,[fj[i]]);
{$else}
      hprintf(ofile,'%s'+LNEND,
      [fv_string( integer(fj.objects[i]),true)]
      );
{$endif}
    end;
  end;
end;

{****************************************************************************
****************************************************************************}
//returns true if execed.
function execedir:boolean;
begin

  result := false;
  if Goptions.edir_exec=nil then
  exit;

  maybe_fix_exec_state;
  result := execform.execute(Goptions.edir_exec );

  if not Goptions.edir_selectanyway then
  Goptions.edir_exec.r.failure_select := Goptions.edir_name;
end;

{****************************************************************************
function custom_compare(List: TStringList; Index1, Index2: Integer): Integer;
begin
  result := CompareText(list[Index1],list[Index2]);
end;


function custom_case(List: TStringList; Index1, Index2: Integer): Integer;
begin
  result := CompareStr(list[Index1],list[Index2])
end;

But note that for this nonsense, we really WANT the thing compare without
case! Ha la! Even in Linux.... I.e., despite stupid Linux DON'T want

  bingo.bak
  bongo.BAK

to wind-up in different places....

****************************************************************************}
{$ifdef FIX_SORT}
const
xcount :integer = 5;

function ext_compare(
List: TStringList; Index1, Index2: Integer): Integer;
var
ex1, ex2:string;

//make it sort before anything else.
  function fakeempty(s:string):string;
  begin
    if (s='') then
    result := StringOfChar(#1,xcount)
    else
    result := s+StringOfChar(#1,xcount-length(s));
  end;
//making the extensions the same length "fixed" the sort.

begin
//result := CompareText(list[Index1],list[Index2]);

//  dbg_nolog('in ext_compare');

  with fv do begin

    ex1 := fakeempty(Tos_f(fls.objects[integer(List.objects[Index1])]).ext);
    ex2 := fakeempty(Tos_f(fls.objects[integer(List.objects[Index2])]).ext);

//    result := CompareText(ex1,ex2);

//    if result=0 then

    result :=
    CompareText
    (
    ex1 +
    Tos_f(fls.objects[integer(List.objects[Index1])]).name

    ,

    ex2 +
    Tos_f(fls.objects[integer(List.objects[Index2])]).name


    );

  end;

end;
{$endif}

{****************************************************************************
+ Wed 9/05/2007 10:38 am. Fold dot files better into ediros? (i.e. get out of
loop) That is, for every ediros, *start* with an fj that either has or doesn't
have ".*"? I'd have to

  1. sort the thing so I can use find.
  2. Search for ".*".
  3. If it *should* throw-away ".*", leave it in,

Or something. See it's not that simple, eh? Like

  1. Determine whether SHOULD throw away ".*".
  2. If should, search for ".*"; if not there, add. If there, OK.
  3. If shouldn't, search for ".*"; if not there, OK. If there, delete.

****************************************************************************}
procedure fixdotjunk;
var
found:boolean;
index:integer;
const
dotmask = '.*';

begin

  if not Goptions.edir_dot then
  exit;
  //the dotmask will have never contaminated the skiplist.

//ok we've been advised to skip .*.

  Goptions.edir_skip_list.sort;
  found := Goptions.edir_skip_list.find(dotmask,index);

//but there could be a file detection thing.
  if FileExists_unq(Goptions.edir_seedot) then begin
    //he wants us to see them in *this* directory.
    if found then
    Goptions.edir_skip_list.delete(index);
  end else begin
    //he doesn't want us to see them.
    if not found then
    Goptions.edir_skip_list.add(dotmask);
  end;

end;

{****************************************************************************
Tue 9/04/2007 2:08 pm. So my problem here is I wrote this

1. To get new names

2. To get the old names and *add* new names.

In fact, the #2 is more common, and I've screwed things up by sorting
the lists to start-with for #1!

So I'm going to try to hack everything the FIX_SORT conditional, the general
idea begin

1. Sort the names so I can use "find" instead of "indexof".

2. Then, for the remainder output -- what we output AFTER reading the edir.os
file with existing names -- sort them however I want.

//so the problem is, apparently find DOESN'T work. Of course. It's Borland.

****************************************************************************}

//returns true if it execed.
function edir(thisfv:Tfv):boolean;
var
bk,
er,
dst : string;
label
BADNESS, BADNESS2;

begin
  result := false;

  xinit;

  fixdotjunk;

  ifile := -1;
  ofile := -1;
  er := '';

  dst := os_temporary(true);

  fv := thisfv;

  fj := jgo_slist;

//prepare this valuable value.
  dontseedots :=
  Goptions.edir_dot and
  (not FileExists_unq(Goptions.edir_seedot));

{$ifdef FIX_SORT}
  getosnames;
{$else}
//get the names from OS fileview. They are stuck in TStringList fj.
  if (Goptions.edir_ext) then
  getosexts
  else
  getosnames;

{$endif}  //if FIX_SORT.

{$ifdef LINUX}
//these apparently have to be right for TstringList.find to work.
  fj.casesensitive := true;
{$endif}
{$ifdef MSWINDOWS}
  fj.casesensitive := false;
{$endif}


{$ifndef FIX_SORT}
//we leave them unsorted in any case for EDIR_eXt.
  if (Goptions.edir_ext) then begin
    exts_names;   //get the names back instead of the extensions.
  end;
//we don't actually use those things -- except for searching for the
//names in processedir. //And output_remainders!
{$endif}

  er := safe_format('couldn''t open temporary output file %s',
  [dst]);

  //no ifile is not an error.

  ifile := FileOpen(Goptions.edir_name,fmOpenRead);
  if (ifile>=0) then begin
    ofile := StupidNewFile(dst);
    if ofile<0 then
    goto BADNESS;

{$ifdef FIX_SORT}
    stupid_string_sort(fj); //this is actually faster! 'Cause ascii.
    fj.sorted := true;  //but won't do find without this....
{$endif}

    processedir;
  end;

  if ofile<0 then
  ofile := StupidNewFile(dst);
  if ofile<0 then
  goto BADNESS;

{$ifdef FIX_SORT}
{****************************************************************************
NOW sort the REMAINDERS *ONLY IF* extension -- which of course is usually
true, at least here at the laboratories.
****************************************************************************}

  if Goptions.edir_ext then begin

    xcount := Goptions.ext_field+1;
    //because I sort with the period.

    fj.sorted := false; //Won't do customsort without it.
//Brilliant helpful really smart borland.

    fj.customsort(ext_compare);

  end;

{$endif}

  output_remainders;

  er := '';

  BADNESS:
  if (ifile>=0) then
  FileClose(ifile);
  if (ofile>=0) then
  FileClose(ofile);

  fj.free;

  if length(er)>0 then begin
    errorf('edir error: %s',[er]);
    goto BADNESS2;
  end;

  bk := bakname(Goptions.edir_name,Goptions.edir_bak);
  DeleteFile(bk);
  if not rename_er(Goptions.edir_name,bk,true) then
  goto BADNESS2;

  if not rename_er(dst,Goptions.edir_name) then
  goto BADNESS2;

  result :=
  execedir;

  BADNESS2:

end;

end.

