{osinfo.pas

See the bottom of main.pas for the BSD-style license.

}

{$I osops.i}

unit osinfo;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QButtons, QExtCtrls;

type
  Tosinfof = (osinfo_null, osinfo_deny, // added Monday, July 17, 2006 5:16 pm.
  osinfo_info, osinfo_help,
  osinfo_confirm, osinfo_error);
  {input and output type.

  osinfo_info: just shows some info, waits for OK press.

  osinfo_help: the same as osinfo_info, except a help button is present, and
  if pressed, the function returns osinfo_help; otherwise it returns
  osinfo_info.

  osinfo_confirm: shows Yes No buttons, and returns osinfo_confirm if the Yes
  button it pressed. Otherwise, osinfo_info.

  }

  TFosinfo = class(TForm)
    OkButton: TBitBtn;
    HelpButton: TBitBtn;
    Image_info: TImage;
    TextLabel: TLabel;
    NoButton: TBitBtn;
    Image_error: TImage;
    ClipButton: TBitBtn;
    procedure HelpButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure NoButtonClick(Sender: TObject);
    procedure ClipButtonMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
    inforesult : Tosinfof;
    is_showing : boolean;
    ClipShift : TShiftState;
  end;

var
  Fosinfo: TFosinfo;

function osinfof(
const TheFormat: string;
const Args:array of const;
what : Tosinfof = osinfo_info):Tosinfof; overload;

function osinfof(
const TheFormat: string;
what : Tosinfof = osinfo_info):Tosinfof; overload;

//doesn't show.
procedure osinf(
const TheFormat: string;
const Args:array of const;
what : Tosinfof = osinfo_info);

implementation

{$R *.xfm}

uses qclipbrd, code, sbrs, qt, options, fontfun;

procedure osinf(
const TheFormat: string;
const Args:array of const;
what : Tosinfof = osinfo_info);
begin
  unsplash;

  with Fosinfo do begin
    inforesult := osinfo_info;
    TextLabel.caption := safe_format(Theformat,args);
    Image_info.visible := false;
    Image_error.visible := false;
    case what of
      osinfo_help: begin
        caption := 'info';
        OKbutton.caption := 'OK';
        OKButton.visible := true;
        Nobutton.visible := false;
        HelpButton.visible := true;
        Image_info.visible := true;
      end;

      osinfo_info: begin
        caption := 'info';
        OKbutton.caption := 'OK';
        OKButton.visible := true;
        Nobutton.visible := false;
        HelpButton.visible := false;
        Image_info.visible := true;
      end;

      osinfo_confirm: begin
        caption := 'confirmation';
        OKbutton.caption := '&Yes';
        OKButton.visible := true;
        Nobutton.visible := true;
        HelpButton.visible := false;
        Image_info.visible := true;
      end;

      osinfo_error: begin
        caption := 'error';
        OKButton.caption := 'whatever ...';
        OKButton.visible := true;
        Nobutton.visible := false;
        Image_error.visible := true;
      end;

    end;
//    showmodal;
//    result := inforesult;
    inforesult := osinfo_null;
  end;
end;

function osinfof(
const TheFormat: string;
const Args:array of const;
what : Tosinfof = osinfo_info):Tosinfof;
begin
  osinf(TheFormat,Args,what);
  Fosinfo.showmodal;
  result := Fosinfo.inforesult;
end;

procedure TFosinfo.HelpButtonClick(Sender: TObject);
begin
  inforesult := osinfo_help;
  close;
end;

procedure TFosinfo.ClipButtonMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (ssShift in Shift) then
  clipboard.astext := makelineend(clipboard.astext)
  + '-------------------------' + LNEND + TextLabel.Caption
  else
  clipboard.astext := TextLabel.caption;
end;

procedure TFosinfo.FormCreate(Sender: TObject);
begin
  autoscroll := true;
end;

procedure TFosinfo.OkButtonClick(Sender: TObject);
begin
  inforesult := osinfo_confirm;
end;

procedure TFosinfo.NoButtonClick(Sender: TObject);
begin
  inforesult := osinfo_deny;
end;

procedure TFosinfo.FormShow(Sender: TObject);
begin
  fixfonts(fosinfo);
  formguard(fosinfo);

  is_showing := true;
  OkButton.setfocus;

end;

procedure TFosinfo.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key=key_escape then
  modalresult := mrCancel;
end;

function osinfof(
const TheFormat: string;
what : Tosinfof = osinfo_info):Tosinfof;
begin
  result :=
  osinfof(TheFormat,[0],what);
end;



procedure TFosinfo.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  is_showing := false;
end;

end.
