{publics.pas

See the bottom of main.pas for the BSD-style license.

}

{$I osops.i}

unit publics;

interface

uses qcomctrls, qstdctrls, qextctrls, sysutils, classes, qgraphics
{$ifdef LINUX}
,Libc
{$endif}
;

var
Gos_delete_dontsave : boolean;
{a nasty global used to tell os_delete not to save to del_dir --
temporarily we hope.}

type

jcopy = (jcopy_NADA, jcopy_OVERWRITE, jcopy_NEWER, jcopy_SAME);

const
s_jcopy_OVERWRITE = 'overwrite';
s_jcopy_NEWER = 'newer';
s_jcopy_SAME = 'same';

function jcopy_string_value(s:string):jcopy;

type

Rquick_copy = set of (quick_valid, quick_quotes, quick_slash, quick_name,
quick_cute);

Tclip_number = class    {used in clipform.}
  index : integer;
end;

Tos_key = record
  show : string;
  key : word;
  shift : Tshiftstate;
end;

Tsearch_kind = (search_CONTAINS, search_BEGINS_WITH, search_EXACT,
search_EXACT_SLASH //used internally in Ttree.find.

);

procedure stringListAssign(var dst,src:TStringList);

type

Rexec_assignable = record
  name,
  exec,
  argfile : string;
  key : Tos_key;
  showing_prompt,
  log,
  invisible,
  is_fake_blank,        {set/used by code in exec}
  in_dir,in_files,
  is_enter, {set by options if key is enter. }
  doubleclick,
  full, prompt, short, quotes, modal,
  args, select, marked:boolean;
  reread_enable:boolean;
  rereadwait,
  re_mumbo_its, re_mumbo_wait,
  all_reread,     //this is the numeric specificatin for reread.
  non_modaldelay : integer;
  throttling : boolean;
  failure_select,
  //if exec fails, select the file if not ''.
  directory : string; {added Monday, February 2, 2004 1:48 pm.}
  ignore_reread: boolean;   //edir thing uses this at least...
end;

Texec = class
  r : Rexec_assignable;
  fixed_arglist:TStringList;
  xops : TStringList;
  {they get treated as arguments, i.e. "-x" etc.}
  reread : TStringList;
{****************************************************************************
Mon 4/30/2007 11:26 am. this, however, is a list of something or other that
somehow specififies the rereads for each tree, or linux start-of-dir, with the
numeric value somehow stored in the object of the string in the reread list.
****************************************************************************}

  constructor Create(dontinit : boolean = false);
  function S_xops:string;
  procedure L_xops(L:TStringList);
  function get_reread:integer;
  destructor Destroy; override;

  procedure assign(source:Texec);
{****************************************************************************
IMPLEMENTING ASSIGN                                   Wed 10/03/2007 2:37 pm

Assign has to be maintained with any change in Texec. // unless EXEC_ASSIGN
scheme is in effect; then all I have to do is make sure that anything *not* in
r:Rexec_assignable is maintained. Fields may be added freely to
Rexec_assignable, but taking care to make sure they are copyable -- of course
I still have to make sure I get that right. ... Of course this whole thing is
silly, but it was just so untidy doing the assignment the other way....

C--, on the other hand, by default copies the "record" part of an object with
simple assignment; if the constructor for the class allocates anything, then
you write a copy constructor to handle those things. I.e., it does what I've
foolishly contrived here automatically. ... Much tidier I must concede.

PATENTABLE!

So I looked around, and this idea of using a copyable object to help implement
the assign method with Delphi is apparently unique! ... So to summarize, the
idea is

1. Define a record type with all the copyable things you want in your class.

2. Define an instance of that record in your class.

3. Everything else -- things that need "deep-copy" create() stuff -- goes into
the fields of the class.

4. Then in your assign method, just copy the object -- which Delphi is
copacetic about -- and do whatever fancier manipulation you need with the
other class fields.

5. Inheriting such a class and continuing the scheme would presumably involve
something like two objects, which isn't pretty -- but then the whole thing is
a kludge....

TPERSISTENT

And incidentally, Tpersistent was no help at all; apparently it's just some
kind of tag that includes an "Assign" method, but doesn't actually implement
any code or copying; that's still up to you. It might facilitate file
save/restore somehow....

****************************************************************************}

end;


const
rostring =
{$ifdef MSWINDOWS}
'r/o, sys, or hidden'
{$endif}
{$ifdef LINUX}
'r/o'
{$endif}
;

QUOTES_DEFAULT =
{$ifdef MSWINDOWS}
true
{$endif}
{$ifdef LINUX}
false
{$endif}
;

OwenShowID = 'OwenShow';
//used by KylixCancelStart, and as the default main caption. And elsewhere
//for program identification. Although hardly observed rigorously.

fontsize_minuslimit = -100;
fontsize_getdef=fontsize_minuslimit-1;
{works with main fontsize q.v.}

{my keys. The values will become dangerous and mad at the moment Delph 8
appears. Or before. }
jgok_eq_pl = $8000;
eq_pl_string = '+/=';

type

Tos_f = class {record} {will it speed-up sorts? Ought to...}
  dir:boolean;
  mark : char;
  name,nameroot,ext:string;
  time,attr:integer;
  size : int64;
{$ifdef LINUX}
  atpix : string[6];
  Mode: mode_t;
{$endif}
{$ifdef MSWINDOWS}
  atpix : string[4];
{$endif}
end;

osystem = (NoOS, w9x, XP, LINUX);

const

Selcolor = clBase;
unselcolor = clSilver;
UnsortedColor = clYellow;

SelText = clBlack;
unSelText = clDkGray;


type

fileaction = (xcopy, xmove, xcreate, xrename, xdelete);

Gcheckback = function(story:string; howmuch:integer = 1):boolean;
{if true, give up.}

var
Gcheck : Gcheckback;

type

EUserAnnoyed = class(Exception);
{$ifdef LINUX}
ERenameExists = class(Exception);
{$endif}
EScanFailed = class(Exception);

whichpanel = (none, d, f1, f2);

FileSortKind = (
nada_sort,
all_unsorted,
name_ascend, name_descend,
ext_ascend,  ext_descend,
size_ascend, size_descend,
time_ascend, time_descend,
atts_ascend, atts_descend );

{****************************************************************************
****************************************************************************}

ResultAction = (raWRECK, raOK, raNFG, raABORT,
ra_Same_Overwrite_OK,  {seemed to be same file, but user wanted copy anyway.}
raNA);
{raNA means the condition tested for wasn't present. raOK
means it was present, but the user didn't mind.}

const

defaultsort = ext_ascend;

{values for the caption field in the files screen.}
ftagNOTHING = ' ';
ftagCK = '+';
ftagCKED = '*';
ftagSKIPPED = '-';

{as it worked out conveniently, the *caption* is the status thingey
as above; all the info is in the subitems. Here are their indices.}
filesub = 0;
//sizesub = 1;
//timesub = 2;  {see we don't use those they're just strings anyway.]
attssub = 3;

{columns in the Tlistview.columns[] thingey. }
nadacolumn = 0;
namecolumn = 1;
sizecolumn = 2;
timecolumn = 3;
attscolumn = 4;

ascending = '+';
descending = '-';

namecaption : string = 'name';
extcaption  : string = 'ext';
sizecaption : string = 'size';
timecaption : string = 'time';
attscaption = 'atts';

implementation

{procedure DelphiWants_it;
begin
end;}

uses code, main, sbrs , options ;

constructor Texec.Create(dontinit : boolean = false);
begin
  inherited Create;
  {code goes after inherited create}

  with r do begin
    showing_prompt := true;

  {****************************************************************************
  Wed 8/15/2007 12:31 pm. The bug was it was @G executing stuff like a batch
  file in the files directory without asking; this fixed it. The excuse is
  showing_prompt changed the behavior of exec in ways I don't understand, but it
  was intended to always be true unless the user specified the magic value
  "prompt=no" -- i.e. as opposed to just "prompt=0" -- so the presumption is
  that it should be true no matter. For some reason, however, the exec code
  creates its blank and empty Texecs with dontinit==true. ... Whatever....
  ****************************************************************************}

    if dontinit then
    exit;

  { put )(*&)(*& defaults in here. options getexecs does the hard work.}
    in_dir := true;
    in_files := true;
    prompt := true;

    quotes := QUOTES_DEFAULT;   //deffed above.

    modal := false;
    all_reread := Goptions.reread;
    rereadwait := Goptions.rereadwait;
    reread_enable := Goptions.reread_enable;

    reread := jgo_slist;    //make a list.
    args := true;
    non_modaldelay := Goptions.non_modaldelay;

    fixed_arglist := nil;
  end;

end;

function Texec.S_xops:string;
var
i:integer;
space:string;
begin
  result := '';

  space := '';

  for i := 0 to xops.count-1 do begin
    result := result + space + xops[i];
    space := ' ';
  end;

end;

procedure Texec.L_xops(L:TStringList);
var
i:integer;
begin
  for i:= 0 to xops.count-1 do begin
    L.add(xops[i]);
  end;
end;

destructor Texec.destroy;
begin
  freeandnil(xops);
  freeandnil(fixed_arglist);
  freeandnil(reread);

  inherited destroy;
end;

{$ifdef LINUX}
function Texec.get_reread:integer;
var
i:integer;
dir:string;
LABEL
DOIT;
begin
  if reread=nil then begin
    result :=r.all_reread; //he specified a number.
    exit;
  end;

  dir := thisdir_string;

  for i := 0 to reread.count-1 do begin
    if dir_begins_with(reread[i],dir) then
    goto DOIT;
  end;

  i := reread.indexof('otherwise');

  if (i<0) then begin
    result := r.all_reread;  //default reread
    exit;
  end;

  DOIT:
  result :=
  integer(reread.objects[i]);

end;

{$endif}

{$ifdef MSWINDOWS}
function Texec.get_reread:integer;
var
i:integer;
begin
  if reread=nil then begin
    result := r.all_reread; //he specified a number.
    exit;
  end;

  i := reread.indexof(thetree.name);
  if (i<0) then //not this tree
  i := reread.indexof('otherwise');

  if (i<0) then begin
    result := r.all_reread;  //default reread
    exit;
  end;

  result :=
  integer(reread.objects[i]);

end;
{$endif}

{****************************************************************************
Tue 10/02/2007 11:03 am. So sadly there is no blanket object copy in Delphi.
It would've been helpful to at least copy all the junk. But then again.....
****************************************************************************}

//gotta be vars so I can nillify dst.
procedure stringListAssign(var dst,src:TStringList);
begin
  if src=nil then begin
    FreeAndNil(dst);
    exit;
  end;

  if dst=nil then
  dst := jgo_slist;

  dst.assign(src);

end;

procedure Texec.assign(source:Texec);
begin
  self.r := source.r; //see one fell swoop!
  stringListAssign(self.fixed_arglist,source.fixed_arglist);
  stringListAssign(self.xops,source.xops);
  stringListAssign(self.reread,source.reread);
end;

{****************************************************************************
****************************************************************************}

function jcopy_string_value(s:string):jcopy;
begin
  if sametext(s,s_jcopy_OVERWRITE) then
  result := jcopy_OVERWRITE
  else if sametext(s,s_jcopy_NEWER) then
  result := jcopy_NEWER
  else if sametext(s,s_jcopy_SAME) then
  result := jcopy_SAME
  else
  result := jcopy_NADA;

end;

end.

